searchNodes=[{"doc":"IP lookup, with longest prefix match, for IPv4, IPv6 prefixes (and others). Iptrie manages multiple Radix trees, one for each type of Pfx.t/0 prefix used as determined by their maxlen property. That way, IPv4 prefixes ( maxlen: 32 ) use a different radix tree as opposed to e.g. IPv6 ( maxlen: 128 ). Iptrie has a bias towards IPv4 and IPv6 since it uses Pfx to convert arguments to a Pfx.t/0 struct. So, doing other types of prefixes will require the actual Pfx.t/0 structs as arguments for the various Iptrie functions. Like Pfx , Iptrie tries to mirror the representation of results to the argument(s) given, if possible. IPv4/IPv6 iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.2.3.0/24&quot; , &quot;v4&quot; ) ...&gt; |&gt; put ( &quot;128.0.0.0/8&quot; , &quot;v4-128&quot; ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; ) ...&gt; |&gt; put ( &quot;acdc:1978::/32&quot; , &quot;Powerage&quot; ) ...&gt; |&gt; put ( &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; ) ...&gt; |&gt; put ( &quot;::/0&quot; , &quot;no dynamite&quot; ) iex&gt; iex&gt; lookup ( ipt , &quot;1.2.3.128&quot; ) { &quot;1.2.3.0/24&quot; , &quot;v4&quot; } iex&gt; lookup ( ipt , &quot;acdc:1975::&quot; ) { &quot;acdc:1975:0:0:0:0:0:0/32&quot; , &quot;T.N.T&quot; } iex&gt; iex&gt; # separate trees, separate default routes iex&gt; lookup ( ipt , &quot;10.11.12.13&quot; ) { &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; } iex&gt; lookup ( ipt , &quot;abba::&quot; ) { &quot;0:0:0:0:0:0:0:0/0&quot; , &quot;no dynamite&quot; } iex&gt; iex&gt; # visualize the IPv4 &amp; IPv6 radix trees iex&gt; kv32 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 32 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 32 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv4&quot; , kv_tostr : kv32 ) ...&gt; |&gt; ( &amp; File . write ( &quot;img/ipv4.dot&quot; , &amp;1 ) ) . ( ) iex&gt; kv128 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 128 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 128 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv6&quot; , kv_tostr : kv128 ) ...&gt; |&gt; ( &amp; File . write ( &quot;img/ipv6.dot&quot; , &amp;1 ) ) . ( ) Where the radix trees for the IP prefixes look like: Others Iptrie can also be used to do longest prefix match lookup for other types of prefixes, like e.g. MAC addresses: iex&gt; ipt = new ( ) ...&gt; |&gt; put ( % Pfx { bits : &lt;&lt; 0x00 , 0x22 , 0x72 &gt;&gt; , maxlen : 48 } , &quot;American Micro-Fuel Device&quot; ) ...&gt; |&gt; put ( % Pfx { bits : &lt;&lt; 0x00 , 0xd0 , 0xef &gt;&gt; , maxlen : 48 } , &quot;IGT&quot; ) ...&gt; |&gt; put ( % Pfx { bits : &lt;&lt; 0x08 , 0x61 , 0x95 &gt;&gt; , maxlen : 48 } , &quot;Rockwell Automation&quot; ) iex&gt; iex&gt; lookup ( ipt , % Pfx { bits : &lt;&lt; 0x00 , 0xd0 , 0xef , 0xaa , 0xbb &gt;&gt; , maxlen : 48 } ) { % Pfx { bits : &lt;&lt; 0x00 , 0xd0 , 0xef &gt;&gt; , maxlen : 48 } , &quot;IGT&quot; } iex&gt; iex&gt; # longest match for partial prefix iex&gt; lookup ( ipt , % Pfx { bits : &lt;&lt; 0x08 , 0x61 , 0x95 , 0x01 &gt;&gt; , maxlen : 48 } ) |&gt; elem ( 1 ) &quot;Rockwell Automation&quot; iex&gt; iex&gt; kv48 = fn { _k , v } -&gt; &quot; \#{ v } &quot; end iex&gt; radix ( ipt , 48 ) ...&gt; |&gt; Radix . dot ( label : &quot;MAC OUI&quot; , kv_tostr : kv48 ) ...&gt; |&gt; ( &amp; File . write ( &quot;img/mac.dot&quot; , &amp;1 ) ) . ( ) Iptrie does not automatically recognize MAC addresses in string format (like 00-D0-EF-AA-BB-CC-DD-EE or 00:D0:EF:AA:BB:CC:DD:EE ), so the actual Pfx.t/0 structs must be used in the various IPtrie functions. Since prefixes are stored in specific radix trees based on the maxlen of given prefix, you could also mix IPv4, IPv6 and MAC prefixes and possibly others, in a single Iptrie.","ref":"Iptrie.html","title":"Iptrie","type":"module"},{"doc":"Delete one or more prefix, value-pairs from the trie using an exact match. The list of prefixes to delete may contain all types , so all sorts of prefixes can be deleted from multiple radix trees in one go. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) iex&gt; iex&gt; lookup ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.0/24&quot; , &quot;one&quot; } iex&gt; iex&gt; Map . get ( ipt , 32 ) |&gt; Radix . keys ( ) [ &lt;&lt; 1 , 1 , 1 &gt;&gt; , &lt;&lt; 2 , 2 , 2 &gt;&gt; ] iex&gt; iex&gt; ipt = delete ( ipt , &quot;1.1.1.0/24&quot; ) iex&gt; iex&gt; lookup ( ipt , &quot;1.1.1.1&quot; ) nil iex&gt; iex&gt; Map . get ( ipt , 32 ) |&gt; Radix . keys ( ) [ &lt;&lt; 2 , 2 , 2 &gt;&gt; ] iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) ...&gt; |&gt; put ( &quot;abba:1973::/32&quot; , &quot;Ring Ring&quot; ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; ) iex&gt; iex&gt; ipt = delete ( ipt , [ &quot;1.1.1.0/24&quot; , &quot;abba:1973::/32&quot; ] ) iex&gt; iex&gt; Map . get ( ipt , 32 ) |&gt; Radix . keys ( ) [ &lt;&lt; 2 , 2 , 2 &gt;&gt; ] iex&gt; iex&gt; Map . get ( ipt , 128 ) |&gt; Radix . keys ( ) [ &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; ]","ref":"Iptrie.html#delete/2","title":"Iptrie.delete/2","type":"function"},{"doc":"Fetches the prefix,value-pair for given prefix from trie (exact match). In case of success, returns {:ok, {prefix, value}}. If prefix is not present, returns {:error, :notfound} . In case of encoding errors for prefix , returns {:error, :notprefix} Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) iex&gt; iex&gt; fetch ( ipt , &quot;1.1.1.0/24&quot; ) { :ok , { &quot;1.1.1.0/24&quot; , &quot;one&quot; } } iex&gt; iex&gt; fetch ( ipt , &quot;12.12.12.12&quot; ) { :error , :notfound } iex&gt; iex&gt; fetch ( ipt , &quot;13.13.13.333&quot; ) { :error , :notprefix }","ref":"Iptrie.html#fetch/2","title":"Iptrie.fetch/2","type":"function"},{"doc":"Fetches the prefix,value-pair for given prefix from trie (exact match). In case of success, returns {prefix, value}. If prefix is not present, raises a KeyError . If prefix could not be encoded, raises an ArgumentError . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;10.10.10.0/24&quot; , &quot;ten&quot; ) ...&gt; |&gt; put ( &quot;11.11.11.0/24&quot; , &quot;eleven&quot; ) iex&gt; iex&gt; fetch! ( ipt , &quot;10.10.10.0/24&quot; ) { &quot;10.10.10.0/24&quot; , &quot;ten&quot; } iex&gt; iex&gt; fetch! ( ipt , &quot;12.12.12.12&quot; ) ** (KeyError) prefix &quot;12.12.12.12&quot; not found iex&gt; ipt = new ( ) iex&gt; fetch! ( ipt , &quot;13.13.13.333&quot; ) ** (ArgumentError) invalid prefix &quot;13.13.13.333&quot;","ref":"Iptrie.html#fetch!/2","title":"Iptrie.fetch!/2","type":"function"},{"doc":"Returns a new Iptrie, keeping only the entries for which fun returns truthy . The signature for fun is (key, maxlen, value -&gt; boolean), where the (radix) key is the original bitstring of the prefix of type maxlen, used to store some value in that particular radix tree in given trie . Radix trees that are empty, are removed from the new Iptrie. Note that, if need be, Pfx.new(key, maxlen) reconstructs the original prefix used to store the value in the trie . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;rock&quot; ) ...&gt; |&gt; put ( &quot;acdc:1976::/32&quot; , &quot;rock&quot; ) ...&gt; |&gt; put ( &quot;abba:1975::/32&quot; , &quot;pop&quot; ) ...&gt; |&gt; put ( &quot;abba:1976::/32&quot; , &quot;pop&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;v4&quot; ) iex&gt; iex&gt; filter ( ipt , fn _bits , maxlen , _value -&gt; maxlen == 32 end ) ...&gt; |&gt; to_list ( ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , &quot;v4&quot; } ] iex&gt; iex&gt; filter ( ipt , fn _bits , _max , value -&gt; value == &quot;rock&quot; end ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { pfx , value } -&gt; { &quot; \#{ pfx } &quot; , value } end ) [ { &quot;acdc:1975:0:0:0:0:0:0/32&quot; , &quot;rock&quot; } , { &quot;acdc:1976:0:0:0:0:0:0/32&quot; , &quot;rock&quot; } ]","ref":"Iptrie.html#filter/2","title":"Iptrie.filter/2","type":"function"},{"doc":"Finds a prefix,value-pair for given prefix from trie (longest match). In case of success, returns {:ok, {prefix, value}}. If prefix had no match, returns {:error, :notfound} . In case of encoding errors for prefix , returns {:error, :notprefix} Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) iex&gt; iex&gt; find ( ipt , &quot;1.1.1.0/24&quot; ) { :ok , { &quot;1.1.1.0/24&quot; , &quot;one&quot; } } iex&gt; iex&gt; find ( ipt , &quot;12.12.12.12&quot; ) { :error , :notfound } iex&gt; iex&gt; find ( ipt , &quot;13.13.13.333&quot; ) { :error , :notprefix }","ref":"Iptrie.html#find/2","title":"Iptrie.find/2","type":"function"},{"doc":"Finds a prefix,value-pair for given prefix from trie (longest match). In case of success, returns {prefix, value}. If prefix had no match, raises a KeyError . If prefix could not be encoded, raises an ArgumentError . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;10.10.10.0/24&quot; , &quot;ten&quot; ) ...&gt; |&gt; put ( &quot;11.11.11.0/24&quot; , &quot;eleven&quot; ) iex&gt; iex&gt; find! ( ipt , &quot;10.10.10.0/24&quot; ) { &quot;10.10.10.0/24&quot; , &quot;ten&quot; } iex&gt; iex&gt; find! ( ipt , &quot;12.12.12.12&quot; ) ** (KeyError) prefix &quot;12.12.12.12&quot; not found iex&gt; ipt = new ( ) iex&gt; find! ( ipt , &quot;13.13.13.333&quot; ) ** (ArgumentError) invalid prefix &quot;13.13.13.333&quot;","ref":"Iptrie.html#find!/2","title":"Iptrie.find!/2","type":"function"},{"doc":"Return one or more prefix,value-pair(s) using an exact match for given prefix(es) . Examples iex&gt; ipt = new ( [ { &quot;1.1.1.0/30&quot; , &quot;A&quot; } , { &quot;1.1.1.0/31&quot; , &quot;B&quot; } , { &quot;1.1.1.0&quot; , &quot;C&quot; } ] ) iex&gt; iex&gt; get ( ipt , &quot;1.1.1.0/31&quot; ) { &quot;1.1.1.0/31&quot; , &quot;B&quot; } iex&gt; iex&gt; # or get a list of entries iex&gt; iex&gt; get ( ipt , [ &quot;1.1.1.0/30&quot; , &quot;1.1.1.0&quot; ] ) [ { &quot;1.1.1.0/30&quot; , &quot;A&quot; } , { &quot;1.1.1.0&quot; , &quot;C&quot; } ]","ref":"Iptrie.html#get/2","title":"Iptrie.get/2","type":"function"},{"doc":"Return all prefixes stored in all available radix trees in trie . The prefixes are reconstructed as Pfx.t/0 by combining the stored bitstrings with the Radix -tree's type.e. maxlen property). Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; keys ( ipt ) [ % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } ] iex&gt; iex&gt; radix ( ipt , 32 ) |&gt; Radix . keys ( ) [ &lt;&lt; 1 , 1 , 1 &gt;&gt; , &lt;&lt; 2 , 2 , 2 &gt;&gt; ]","ref":"Iptrie.html#keys/1","title":"Iptrie.keys/1","type":"function"},{"doc":"Return the prefixes stored in the radix tree(s) in trie for given type . Where type is a single maxlen or a list thereof. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; keys ( ipt , 32 ) [ % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } ] iex&gt; iex&gt; keys ( ipt , 128 ) [ % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } ] iex&gt; iex&gt; keys ( ipt , 48 ) [ ] iex&gt; iex&gt; keys ( ipt , [ 32 , 48 , 128 ] ) [ % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } ]","ref":"Iptrie.html#keys/2","title":"Iptrie.keys/2","type":"function"},{"doc":"Return all the prefix,value-pairs whose prefix is a prefix for the given search prefix . This returns the less specific entries that enclose the given search prefix . Note that any bitstring is always a prefix of itself. So, if present, the search key will be included in the result. If prefix is not present or not valid, or cannot be encoded as an Ipv4 op IPv6 Pfx.t/0 , an empty list is returned. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/30&quot; , &quot;A30&quot; ) ...&gt; |&gt; put ( &quot;1.1.2.0/24&quot; , &quot;B24&quot; ) iex&gt; iex&gt; less ( ipt , &quot;1.1.1.0/30&quot; ) [ { &quot;1.1.1.0/30&quot; , &quot;A30&quot; } , { &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; } , ] iex&gt; less ( ipt , &quot;2.2.2.2&quot; ) [ ]","ref":"Iptrie.html#less/2","title":"Iptrie.less/2","type":"function"},{"doc":"Return the prefix,value-pair, whose prefix is the longest match for given search prefix . Returns nil if there is no match for search prefix . Silently ignores any errors when encoding the given search prefix by returning nil. Example","ref":"Iptrie.html#lookup/2","title":"Iptrie.lookup/2","type":"function"},{"doc":"Return all the prefix,value-pairs where the search prefix is a prefix for the stored prefix. This returns the more specific entries that are enclosed by given search prefix . Note that any bitstring is always a prefix of itself. So, if present, the search prefix will be included in the result. If prefix is not valid, or cannot be encoded as an Ipv4 op IPv6 t:Pfx.t , nil is returned. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/30&quot; , &quot;A30&quot; ) ...&gt; |&gt; put ( &quot;1.1.2.0/24&quot; , &quot;B24&quot; ) iex&gt; iex&gt; more ( ipt , &quot;1.1.1.0/24&quot; ) [ { &quot;1.1.1.0/30&quot; , &quot;A30&quot; } , { &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; } , { &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; } ]","ref":"Iptrie.html#more/2","title":"Iptrie.more/2","type":"function"},{"doc":"Create an new, empty Iptrie. Example iex&gt; Iptrie . new ( ) % Iptrie { }","ref":"Iptrie.html#new/0","title":"Iptrie.new/0","type":"function"},{"doc":"Create a new Iptrie, populated via a list of { prefix/0 , any/0 }-pairs. Example iex&gt; ipt = Iptrie . new ( [ { &quot;1.1.1.0/24&quot; , &quot;net1&quot; } , { &quot;acdc:1975::/32&quot; , &quot;TNT&quot; } ] ) iex&gt; Map . get ( ipt , 32 ) { 0 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;net1&quot; } ] , nil } iex&gt; Map . get ( ipt , 128 ) { 0 , nil , [ { &lt;&lt; 172 , 220 , 25 , 117 &gt;&gt; , &quot;TNT&quot; } ] }","ref":"Iptrie.html#new/1","title":"Iptrie.new/1","type":"function"},{"doc":"Populate the trie with a list of {prefix,value}-pairs. This always uses an exact match for prefix , updating its value if it exists. Any errors are silently ignored as the trie is always returned. Example iex&gt; ipt = new ( [ { &quot;1.1.1.0/24&quot; , 0 } , { &quot;1.1.1.1&quot; , 0 } , { &quot;1.1.1.1&quot; , &quot;x&quot; } ] ) iex&gt; iex&gt; get ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.1&quot; , &quot;x&quot; }","ref":"Iptrie.html#put/2","title":"Iptrie.put/2","type":"function"},{"doc":"Puts value under prefix in the trie . This always uses an exact match for prefix , replacing its value if it exists. Any errors are silently ignored as the tree is always returned. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 0 ) ...&gt; |&gt; put ( &quot;1.1.1.1&quot; , 1 ) ...&gt; |&gt; put ( &quot;1.1.1.1&quot; , &quot;x&quot; ) iex&gt; iex&gt; get ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.1&quot; , &quot;x&quot; }","ref":"Iptrie.html#put/3","title":"Iptrie.put/3","type":"function"},{"doc":"Return the radix tree for given type or a new empty tree. If there is no Radix tree for given type , an empty radix will be returned without storing it in the trie . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; radix ( ipt , 32 ) { 0 , { 6 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 1 } ] , [ { &lt;&lt; 2 , 2 , 2 &gt;&gt; , 2 } ] } , nil } iex&gt; iex&gt; radix ( ipt , 128 ) { 0 , nil , { 18 , [ { &lt;&lt; 172 , 220 , 25 , 117 &gt;&gt; , 3 } ] , [ { &lt;&lt; 172 , 220 , 32 , 33 &gt;&gt; , 4 } ] } } iex&gt; radix ( ipt , 48 ) { 0 , nil , nil }","ref":"Iptrie.html#radix/2","title":"Iptrie.radix/2","type":"function"},{"doc":"Invoke fun on all prefix,value-pairs in all radix trees in the trie . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; reduce ( ipt , 0 , fn _key , value , acc -&gt; acc + value end ) 10 iex&gt; iex&gt; reduce ( ipt , %{ } , fn key , value , acc -&gt; Map . put ( acc , key , value ) end ) %{ &lt;&lt; 1 , 1 , 1 &gt;&gt; =&gt; 1 , &lt;&lt; 2 , 2 , 2 &gt;&gt; =&gt; 2 , &lt;&lt; 172 , 220 , 25 , 117 &gt;&gt; =&gt; 3 , &lt;&lt; 172 , 220 , 32 , 33 &gt;&gt; =&gt; 4 }","ref":"Iptrie.html#reduce/3","title":"Iptrie.reduce/3","type":"function"},{"doc":"Invoke fun on each prefix,value-pair in the radix tree for given type This simply wraps Radix.reduce/3 for the radix tree in trie at given type . The function fun is called with the radix key, value and acc accumulator and should return an updated accumulator. The result is the last acc accumulator returned. If type is a list of type 's, the acc accumulator is updated across all radix trees of type 's. Probably not entirely usefull, but there you go. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; reduce ( ipt , 32 , 0 , fn _key , value , acc -&gt; acc + value end ) 3 iex&gt; reduce ( ipt , 48 , 0 , fn _key , value , acc -&gt; acc + value end ) 0 iex&gt; reduce ( ipt , 128 , 0 , fn _key , value , acc -&gt; acc + value end ) 7 iex&gt; iex&gt; reduce ( ipt , [ 32 , 48 , 128 ] , 0 , fn _key , value , acc -&gt; acc + value end ) 10","ref":"Iptrie.html#reduce/4","title":"Iptrie.reduce/4","type":"function"},{"doc":"Return all prefix,value-pairs from all available radix trees in trie . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; to_list ( ipt ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , 1 } , { % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , 2 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , 3 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } , 4 } ]","ref":"Iptrie.html#to_list/1","title":"Iptrie.to_list/1","type":"function"},{"doc":"Returns the prefix,value-pairs from the radix trees in trie for given type (s). If the radix tree for type does not exist, an empty list is returned. If type is a list of types, a flat list of all prefix,value-pairs of all radix trees of given types is returned. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; to_list ( ipt , 32 ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , 1 } , { % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , 2 } ] iex&gt; to_list ( ipt , [ 32 , 48 , 128 ] ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , 1 } , { % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , 2 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , 3 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } , 4 } ]","ref":"Iptrie.html#to_list/2","title":"Iptrie.to_list/2","type":"function"},{"doc":"Lookup prefix and update the matched entry, only if found. Uses longest prefix match, so search prefix is usually matched by some less specific prefix. If matched, fun is called on its value. If prefix had no longest prefix match, the trie is returned unchanged. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 0 ) ...&gt; |&gt; update ( &quot;1.1.1.0&quot; , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;1.1.1.1&quot; , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;2.2.2.2&quot; , fn x -&gt; x + 1 end ) iex&gt; get ( ipt , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 2 } iex&gt; lookup ( ipt , &quot;2.2.2.2&quot; ) nil","ref":"Iptrie.html#update/3","title":"Iptrie.update/3","type":"function"},{"doc":"Lookup prefix and, if found, update its value or insert the default. Uses longest prefix match, so search prefix is usually matched by some less specific prefix. If matched, fun is called on the entry's value. If prefix had no longest prefix match, the default is inserted and fun is not called. Example iex&gt; ipt = new ( ) ...&gt; |&gt; update ( &quot;1.1.1.0/24&quot; , 0 , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;1.1.1.0&quot; , 0 , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;1.1.1.1&quot; , 0 , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;2.2.2.2&quot; , 0 , fn x -&gt; x + 1 end ) iex&gt; lookup ( ipt , &quot;1.1.1.2&quot; ) { &quot;1.1.1.0/24&quot; , 2 } iex&gt; iex&gt; # probably not what you wanted: iex&gt; iex&gt; lookup ( ipt , &quot;2.2.2.2&quot; ) { &quot;2.2.2.2&quot; , 0 }","ref":"Iptrie.html#update/4","title":"Iptrie.update/4","type":"function"},{"doc":"Return all the values stored in all radix trees in trie . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; values ( ipt ) [ 1 , 2 , 3 , 4 ]","ref":"Iptrie.html#values/1","title":"Iptrie.values/1","type":"function"},{"doc":"Return the values stored in the radix trees in trie for given type . Where type is a either single maxlen or a list thereof. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; values ( ipt , 32 ) [ 1 , 2 ] iex&gt; iex&gt; values ( ipt , 128 ) [ 3 , 4 ] iex&gt; iex&gt; values ( ipt , 48 ) [ ] iex&gt; iex&gt; values ( ipt , [ 32 , 48 , 128 ] ) [ 1 , 2 , 3 , 4 ]","ref":"Iptrie.html#values/2","title":"Iptrie.values/2","type":"function"},{"doc":"A prefix represented as an opague Pfx.t/0 struct, an Pfx.ip_address/0 , Pfx.ip_prefix/0 or CIDR string. See: Pfx .","ref":"Iptrie.html#t:prefix/0","title":"Iptrie.prefix/0","type":"type"},{"doc":"An Iptrie struct that contains a Radix tree per type of Pfx.t/0 used. A prefix' type is determined by its maxlen property: IPv4 has maxlen: 32 , IPv6 has maxlen: 128 , MAC addresses have maxlen: 48 and so on. Although Iptrie facilitates lpm lookups of any type of prefix, it has a bias towards IP prefixes. So, any binaries (strings) are interpreted as CIDR-strings and tuples of address digits and/or {address-digits, length) are interpreted as IPv4 or IPv6 representations.","ref":"Iptrie.html#t:t/0","title":"Iptrie.t/0","type":"type"},{"doc":"README Online Documentation . IP lookup, with longest prefix match, for IPv4, IPv6 prefixes (and others). Iptrie manages multiple Radix trees, one for each type of Pfx.t/0 prefix used as determined by their maxlen property. That way, IPv4 prefixes ( maxlen: 32 ) use a different radix tree as opposed to e.g. IPv6 ( maxlen: 128 ). Iptrie has a bias towards IPv4 and IPv6 since it uses Pfx to convert arguments to a Pfx.t/0 struct. So, doing other types of prefixes will require the actual Pfx.t/0 structs as arguments for the various Iptrie functions. Like Pfx , Iptrie tries to mirror the representation of results to the argument(s) given, if possible.","ref":"readme.html","title":"README","type":"extras"},{"doc":"iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.2.3.0/24&quot; , &quot;v4&quot; ) ...&gt; |&gt; put ( &quot;128.0.0.0/8&quot; , &quot;v4-128&quot; ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; ) ...&gt; |&gt; put ( &quot;acdc:1978::/32&quot; , &quot;Powerage&quot; ) ...&gt; |&gt; put ( &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; ) ...&gt; |&gt; put ( &quot;::/0&quot; , &quot;no dynamite&quot; ) iex&gt; iex&gt; lookup ( ipt , &quot;1.2.3.128&quot; ) { &quot;1.2.3.0/24&quot; , &quot;v4&quot; } iex&gt; lookup ( ipt , &quot;acdc:1975::&quot; ) { &quot;acdc:1975:0:0:0:0:0:0/32&quot; , &quot;T.N.T&quot; } iex&gt; iex&gt; # separate trees, separate default routes iex&gt; lookup ( ipt , &quot;10.11.12.13&quot; ) { &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; } iex&gt; lookup ( ipt , &quot;abba::&quot; ) { &quot;0:0:0:0:0:0:0:0/0&quot; , &quot;no dynamite&quot; } iex&gt; iex&gt; # visualize the IPv4 &amp; IPv6 radix trees iex&gt; kv32 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 32 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 32 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv4&quot; , kv_tostr : kv32 ) ...&gt; |&gt; ( &amp; File . write ( &quot;img/ipv4.dot&quot; , &amp;1 ) ) . ( ) iex&gt; kv128 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 128 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 128 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv6&quot; , kv_tostr : kv128 ) ...&gt; |&gt; ( &amp; File . write ( &quot;img/ipv6.dot&quot; , &amp;1 ) ) . ( ) Where the radix trees for the IP prefixes look like:","ref":"readme.html#ipv4-ipv6","title":"README - IPv4/IPv6","type":"extras"},{"doc":"Iptrie can also be used to do longest prefix match lookup for other types of prefixes, like e.g. MAC addresses: iex&gt; ipt = new ( ) ...&gt; |&gt; put ( % Pfx { bits : &lt;&lt; 0x00 , 0x22 , 0x72 &gt;&gt; , maxlen : 48 } , &quot;American Micro-Fuel Device&quot; ) ...&gt; |&gt; put ( % Pfx { bits : &lt;&lt; 0x00 , 0xd0 , 0xef &gt;&gt; , maxlen : 48 } , &quot;IGT&quot; ) ...&gt; |&gt; put ( % Pfx { bits : &lt;&lt; 0x08 , 0x61 , 0x95 &gt;&gt; , maxlen : 48 } , &quot;Rockwell Automation&quot; ) iex&gt; iex&gt; lookup ( ipt , % Pfx { bits : &lt;&lt; 0x00 , 0xd0 , 0xef , 0xaa , 0xbb &gt;&gt; , maxlen : 48 } ) { % Pfx { bits : &lt;&lt; 0x00 , 0xd0 , 0xef &gt;&gt; , maxlen : 48 } , &quot;IGT&quot; } iex&gt; iex&gt; # longest match for partial prefix iex&gt; lookup ( ipt , % Pfx { bits : &lt;&lt; 0x08 , 0x61 , 0x95 , 0x01 &gt;&gt; , maxlen : 48 } ) |&gt; elem ( 1 ) &quot;Rockwell Automation&quot; iex&gt; iex&gt; kv48 = fn { _k , v } -&gt; &quot; \#{ v } &quot; end iex&gt; radix ( ipt , 48 ) ...&gt; |&gt; Radix . dot ( label : &quot;MAC OUI&quot; , kv_tostr : kv48 ) ...&gt; |&gt; ( &amp; File . write ( &quot;img/mac.dot&quot; , &amp;1 ) ) . ( ) Iptrie does not automatically recognize MAC addresses in string format (like 00-D0-EF-AA-BB-CC-DD-EE or 00:D0:EF:AA:BB:CC:DD:EE ), so the actual Pfx.t/0 structs must be used in the various IPtrie functions. Since prefixes are stored in specific radix trees based on the maxlen of given prefix, you could also mix IPv4, IPv6 and MAC prefixes and possibly others, in a single Iptrie.","ref":"readme.html#others","title":"README - Others","type":"extras"},{"doc":"Iptrie can be installed by adding iptrie to your list of dependencies in mix.exs : def deps do [ { :iptrie , &quot;~&gt; 0.3.0&quot; } ] end Repositiory is on github .","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"Changelog 0.3.0 added fetch/2, fetch!/2 added find/2, find!/2 added filter/2 0.2.0 added radix/2 added reduce/3, reduce/4 added to_list/1, to_list/2 0.1.0 initial version","ref":"changelog.html","title":"Changelog","type":"extras"}]